#!/usr/bin/perl

# dup.pl - Monitoriza la interfaz de red en modo promiscuo en busca de SNMP-Traps destinados a una IP dada y los 
#          reenvia a uno o mas destinos. Los paquetes generados atravesaran la cadena OUTPUT de Netfilter con lo
#          cual se puede hacer un posterior DNAT en funcion de la IP origen (-A OUTPUT -t nat -s <ip> -j DNAT)
#
# Autor:           icehouse <icehouse@salix.org>
# Modificaciones:  Roman Medina <roman@rs-labs.com>
#
# Fecha: 02.Mar.2005

use Net::RawIP;
use Net::PcapUtils;
use NetPacket::Ethernet qw(:ALL);
use NetPacket::IP qw(:strip);
use NetPacket::UDP qw(:ALL);
require "getopt.pl";

&Getopt('iodf');

if (!$opt_i || !$opt_o) {
		print "./dup.pl -i <listening IP> -o <dest IP>[,<dest IP>,<dest IP>,...] [-d <interface>] [-f yes]\n";
		exit;
}

my $device= $opt_d;
my $localip= $opt_i;
my @destinos = split /,/, $opt_o;

if ($opt_f eq "yes") {exit if fork}

$SIG{INT} = $SIG{TERM} = \&signal_handler;
sub signal_handler {
        exit 0;
}


sub resend
{		
	my @parametros = @_ ;
	
	foreach (@destinos) {
	

		$a = new Net::RawIP({udp => {}});
		$a->set({
				ip => {
					saddr => @parametros[0],
					frag_off => 0,
					tos => 0,
					daddr => $_
				},
				udp => {
					source => @parametros[2],
					dest => '162', 
					data=> @parametros[1]
				}
			});
			
		$a->send(1,1);
	}
	
}


sub process_pkt {
        my ($arg, $hdr, $pkt) = @_;
        my $eth_obj = NetPacket::Ethernet->decode($pkt);
        my $ip_obj = NetPacket::IP->decode($eth_obj->{data});
	my $udp_obj = NetPacket::UDP->decode($ip_obj->{data});
	
	my @ipdata= ("$ip_obj->{src_ip}","$udp_obj->{data}","$udp_obj->{src_port}");
	
	&resend(@ipdata);


}


Net::PcapUtils::loop( \&process_pkt,
                      DEV=>$device,
                      PROMISC => 1,
                      FILTER => "udp dst port 162 and dst host $localip",
                      SNAPLEN => 1500 );

